/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifdef HAVE_CONFIG_H
    #include "autoconfig.h"
#endif

#include "hwdesc.h"
#include "storage.h"

namespace dSA
{

HWBase::HWBase(dsid_t dsid) : m_dsid(dsid), m_device_type(Unknown) {}

const dsid_t HWBase::getDSID() const
{
    return m_dsid;
}

const device_type HWBase::getDeviceType() const
{
    return m_device_type;
}

HWBase::~HWBase() {}

HWBase *HWBase::clone() const
{
    return new HWBase(*this);
}

dSM::dSM(dsid_t dsid) : HWBase(dsid)
{
    m_device_type = dSMeter;
    m_apartment_id = 0;
}

dSM *dSM::clone() const
{
    return new dSM(*this);
}

void dSM::setPresent(uint8_t present)
{
    m_present = present;
}

uint8_t dSM::getPresent() const
{
    return m_present;
}

void dSM::setHWVersion(uint32_t hw_version)
{
    m_hw_version = hw_version;
}

uint32_t dSM::getHWVersion() const
{
    return m_hw_version;
}

void dSM::setARMSwVersion(uint32_t arm_sw_version)
{
    m_arm_sw_version = arm_sw_version;
}

uint32_t dSM::getARMSwVersion() const
{
    return m_arm_sw_version;
}

void dSM::setDSPSwVersion(uint32_t dsp_sw_version)
{
    m_dsp_sw_version = dsp_sw_version;
}

uint32_t dSM::getDSPSwVersion() const
{
    return m_dsp_sw_version;
}


void dSM::setAPIVersion(uint16_t api_version)
{
    m_api_version = api_version;
}

uint16_t dSM::getAPIVersion() const
{
    return m_api_version;
}

void dSM::setName(std::string name)
{
    m_name = name;
}

std::string dSM::getName() const
{
    return m_name;
}

void dSM::setApartmentId(uint8_t id)
{
    m_apartment_id = id;
}

uint8_t dSM::getApartmentId() const
{
    return m_apartment_id;
}

void dSM::setConfigurationId(uint64_t id)
{
    m_config_hash = id;
}

uint64_t dSM::getConfigurationId() const
{
    return m_config_hash;
}

void dSM::setDeviceCount(uint16_t count)
{
    m_count_devices = count;
}

uint16_t dSM::getDeviceCount() const
{
    return m_count_devices;
}

void dSM::setActiveDeviceCount(uint16_t count)
{
    m_count_active_devices = count;
}

uint16_t dSM::getActiveDeviceCount() const
{
    return m_count_active_devices;
}

void dSM::setInactiveDeviceCount(uint16_t count)
{
    m_count_inactive_devices = count;
}

uint16_t dSM::getInactiveDeviceCount() const
{
    return m_count_inactive_devices;
}

void dSM::setZoneId(uint8_t index, uint16_t id)
{
    if (m_zone_ids.capacity() < (size_t)(index + 1))
        m_zone_ids.reserve(index+1);
    m_zone_ids[index] = id;
}

std::vector<uint16_t> dSM::getZoneIds() const
{
    return m_zone_ids;
}


Device::Device(dsid_t dsid, dsid_t dsm_dsid) : HWBase(dsid),
                                               m_dsm_dsid(dsm_dsid)
{
    m_device_type = dSDevice;
}

Device *Device::clone() const
{
    return new Device(*this);
}

const dsid_t Device::getDSMdSID() const
{
    return m_dsm_dsid;
}

void Device::setId(uint16_t device_id)
{
    m_device_id = device_id;
}

uint16_t Device::getId() const
{
    return m_device_id;
}

void Device::setFunctionId(uint16_t function_id)
{
    m_function_id = function_id;
}

uint16_t Device::getFunctionId() const
{
    return m_function_id;
}

void Device::setProductId(uint16_t product_id)
{
    m_product_id = product_id;
}

uint16_t Device::getProductId() const
{
    return m_product_id;
}

void Device::setVersion(uint16_t version)
{
    m_version = version;
}

uint16_t Device::getVersion() const
{
    return m_version;
}

void Device::setZoneId(uint16_t zone_id)
{
    m_zone_id = zone_id;
}

uint16_t Device::getZoneId() const
{
    return m_zone_id;
}

void Device::setState(uint8_t state)
{
    m_state = state;
}

uint8_t Device::getState() const
{
    return m_state;
}

void Device::setButtonId(uint8_t button_id)
{
    m_button_id = button_id;
}

uint8_t Device::getButtonId() const
{
    return m_button_id;
}

void Device::setLocked(uint8_t locked)
{
    m_locked = locked;
}

uint8_t Device::getLocked() const
{
    return m_locked;
}

void Device::setOutputMode(uint8_t output_mode)
{
    m_output_mode = output_mode;
}

uint8_t Device::getOutputMode() const
{
    return m_output_mode;
}

void Device::setName(const std::string name)
{
    m_name = name;
}

/*
void Device::setName(const char *name)
{
    m_name = name;
}
*/

std::string Device::getName() const
{
    return m_name;
}

void Device::setSerialNumber(uint32_t number)
{
    m_serial_number = number;
}

uint32_t Device::getSerialNumber() const
{
    return m_serial_number;
}

} // namespace
